/*
  LinEAK - Linux support for Easy Access and Internet Keyboards      
  Copyright (C) 2001,2002 Mark Smulders <Mark@PIRnet.nl>
  
  lineakconfig, GTK+ interface for configuring lineakd

  This program is free software; you can redistribute it and/or modify  
  it under the terms of the GNU General Public License as published by  
  the Free Software Foundation; either version 2, or (at your option)   
  any later version.                                                    
                                                                        
  This program is distributed in the hope that it will be useful,       
  but WITHOUT ANY WARRANTY; without even the implied warranty of        
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
  GNU General Public License for more details.                          
                                                                        
  You should have received a copy of the GNU General Public License     
  along with this program; if not, write to the                         
  Free Software Foundation, Inc., 59 Temple Place - Suite 330,          
  Boston, MA 02111-1307, USA.                                           
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <signal.h>

#include "callbacksproto.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

//#include <dmalloc.h>

/**********************************************************************************************/

/* if an other kb brand is selected, erase the keys and update the models */
void on_kbbrandmenuitem_activate (GtkWidget *widget, gpointer data) {
  //if (ignore_callbacks_now) return;
  //ignore_callbacks_now = TRUE;
  if (FirstTimeBrand || myEAK.EAKbrand==NULL || strcasecmp(myEAK.EAKbrand,(char *)data) != 0) {
	FirstTimeBrand = FALSE;
    select_new_kbbrand(widget, data);
  }
  //ignore_callbacks_now = FALSE;
}

/* if an other model is selected, update the keys */
void on_kbmodelmenuitem_activate (GtkWidget *widget, gpointer data) {
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  if (FirstTimeModel || myEAK.EAKbrand==NULL || strcasecmp(myEAK.EAKmodel,(char *)data) != 0) {
	FirstTimeModel = FALSE;
    select_new_kbmodel(widget, data);
  }
  ignore_callbacks_now = FALSE;
}

/* another special action selected, check the radiobutton, and set the data */
void on_specmenuitem_activate (GtkWidget *widget, gpointer data) {
  GtkWidget *specialradio;
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  specialradio = lookup_widget(widget, "radiospecial");
  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(specialradio)))
	return;
  change_selectedkey_command(data);
  ignore_callbacks_now = FALSE;
}

/* when command radiobutton is toggled */
void on_commandradio_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
  GtkWidget *commandentry;
  gchar *entrytext;
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  if (gtk_toggle_button_get_active(togglebutton)) {
	commandentry = lookup_widget(GTK_WIDGET (togglebutton), "commandentry");
	gtk_entry_set_editable(GTK_ENTRY(commandentry), TRUE);
    entrytext = gtk_editable_get_chars(GTK_EDITABLE (commandentry), 0, -1);
	change_selectedkey_command(entrytext);
    g_free(entrytext); 
  }
  ignore_callbacks_now = FALSE;
}

/* when action radiobutton is toggled */
void on_actionradio_toggled (GtkToggleButton *togglebutton, gpointer user_data) {
  GtkWidget *commandentry, *actoptmenu, *actmenu, *actitem;
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  if (gtk_toggle_button_get_active(togglebutton)) {
	commandentry = lookup_widget(GTK_WIDGET (togglebutton), "commandentry");
    gtk_entry_set_editable(GTK_ENTRY(commandentry), FALSE);
	actoptmenu = lookup_widget(GTK_WIDGET(togglebutton), "specials");
	actmenu = gtk_option_menu_get_menu(GTK_OPTION_MENU(actoptmenu));
	actitem = gtk_menu_get_active(GTK_MENU(actmenu));
	gtk_menu_item_activate(GTK_MENU_ITEM(actitem));
  }
  ignore_callbacks_now = FALSE;
}

/* user changed content of command entry */
void on_commandentry_changed (GtkEditable *editable, gpointer user_data) {
  gchar *entrytext;
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  if (changing_text_from_signal) {
	ignore_callbacks_now = FALSE;
	return;
  }
  entrytext = gtk_editable_get_chars(editable,0,-1);
  change_selectedkey_command(entrytext);
  ignore_callbacks_now = FALSE;
}

/* user changed content of cdrom entry */
void on_cdromentry_changed (GtkEditable *editable, gpointer user_data) {
  gchar *entrytext;
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  entrytext = gtk_editable_get_chars(editable,0,-1);
  if (cdromdev!=NULL) {
    if (strcmp(cdromdev,(char *)entrytext) == 0) {
      ignore_callbacks_now = FALSE;
	  return;
	} else
	  free(cdromdev);
  }
  cdromdev = malloc( (strlen(entrytext)+1) *sizeof(char));
  sprintf(cdromdev,"%s",entrytext);
  ignore_callbacks_now = FALSE;
}

/* user changed content of mixer entry */
void on_mixerentry_changed (GtkEditable *editable, gpointer user_data) {
  gchar *entrytext;
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  entrytext = gtk_editable_get_chars(editable,0,-1);
  if (mixdev!=NULL) {
    if (strcmp(mixdev,(char *)entrytext) == 0) {
      ignore_callbacks_now = FALSE;
	  return;
	} else
	  free(mixdev);
  }
  mixdev = malloc( (strlen(entrytext)+1) *sizeof(char));
  sprintf(mixdev,"%s",entrytext);
  ignore_callbacks_now = FALSE;
}

/* don't allow deselection, we don't like an empty selection ;) */
void on_keylist_unselect (GtkList *list, GtkWidget *widget, gpointer user_data) {
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  gtk_list_select_child(list, widget);
  ignore_callbacks_now = FALSE;
}

/* update GUI info on the selected key */
void on_keylist_select (GtkWidget *widget, gpointer data) {
  gint index = -1;
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  if ((index = gtk_list_child_position (GTK_LIST (widget->parent),widget)) != -1 ) {
	changing_text_from_signal = TRUE;
	refresh_keycommand (widget, index);
	changing_text_from_signal = FALSE;
    selected_key = index;
  }
  ignore_callbacks_now = FALSE;
}

/* OK clicked */
void on_okbutton_clicked (GtkButton *button, gpointer user_data) {
  if (on_applybutton_clicked(NULL, NULL))
    cleanexit();
}

/* Apply clicked */
gboolean on_applybutton_clicked (GtkButton *button, gpointer user_data) {
  extern gboolean ApplyUsed;
  GtkWidget *warn;
  gint thepid=0;
  if (ignore_callbacks_now) return FALSE;
  ignore_callbacks_now = TRUE;
  /* get lineakd PID */
  if ((thepid = get_lineakd_pid())==0) {
	// pid file does not exist
	warn = create_warning_window("ERROR: could not find lineakd running! (1)\n"
	                             "LinEAK configuration not changed.");
	gtk_widget_show(warn);
    ignore_callbacks_now = FALSE;
	return FALSE;
  }
  /* first time apply? -> backup conf file */
  if (!ApplyUsed) {
	ApplyUsed = TRUE;
    backup_conffile();
  }
  /* save the changes to a new conf file */
  save_conffile();
  /* rehash lineakd */
  if (kill(thepid, SIGHUP) == -1) {
	restore_conffile();
	// kill -HUP failed, pid from ps check is invalid, (this should *never* happen)
	warn = create_warning_window("ERROR: could not find lineakd running! (2)\n"
	                             "LinEAK configuration not changed.");
	gtk_widget_show(warn);
    ignore_callbacks_now = FALSE;
	return FALSE;
  }
  ignore_callbacks_now = FALSE;
  return TRUE;
}

/* Cancel clicked */
void on_cancelbutton_clicked (GtkButton *button, gpointer user_data) {
  extern gboolean ApplyUsed;
  gint thepid=0;
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  if (ApplyUsed) {
    if ((thepid = get_lineakd_pid())==0) {
      ignore_callbacks_now = FALSE;
	  return;
	} else {
  	  restore_conffile();
      kill(thepid, SIGHUP);
	}
  }
  cleanexit();
  ignore_callbacks_now = FALSE;
}

/* close the alert box */
void on_alert_closebutton_clicked (GtkButton *button, gpointer user_data) {
  GtkWidget *alertwindow;
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  alertwindow = lookup_widget(GTK_WIDGET (button), "warning_window");
  gtk_widget_destroy(GTK_WIDGET(alertwindow));
  ignore_callbacks_now = FALSE;
}

/**********************************************************************************************/

void on_NCwin_close (GtkButton *button, gpointer user_data) {
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  gtk_main_quit();
  ignore_callbacks_now = FALSE;
}

void on_NCsaveconfbutton_clicked (GtkButton *button, gpointer user_data) {
  char *temp;
  if (ignore_callbacks_now) return;
  ignore_callbacks_now = TRUE;
  if ((temp = lookup_kbtype(myEAK.EAKbrand, myEAK.EAKmodel))==NULL) {
	g_print("*** PANIC: keyboard of brand '%s' and model '%s' not found!\n",myEAK.EAKbrand, myEAK.EAKmodel);
	cleanexit();
  } else {
	kbtype = malloc(strlen(temp)*sizeof(char));
	sprintf(kbtype,"%s",temp);
  }
  if (cdromdev == NULL) {
    cdromdev = (char *)malloc(11*sizeof(char));
	sprintf(cdromdev, "%s", "/dev/cdrom");
  }
  if (mixdev == NULL) {
    mixdev = (char *)malloc(11*sizeof(char));
	sprintf(mixdev, "%s", "/dev/mixer");
  }
  initEAK();
  save_conffile();
  gtk_main_quit();
  ignore_callbacks_now = FALSE;
}