/*
  LinEAK - Linux support for Easy Access and Internet Keyboards      
  Copyright (C) 2001,2002 Mark Smulders <Mark@PIRnet.nl>
  
  lineakconfig, GTK+ interface for configuring lineakd

  This program is free software; you can redistribute it and/or modify  
  it under the terms of the GNU General Public License as published by  
  the Free Software Foundation; either version 2, or (at your option)   
  any later version.                                                    
                                                                        
  This program is distributed in the hope that it will be useful,       
  but WITHOUT ANY WARRANTY; without even the implied warranty of        
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
  GNU General Public License for more details.                          
                                                                        
  You should have received a copy of the GNU General Public License     
  along with this program; if not, write to the                         
  Free Software Foundation, Inc., 59 Temple Place - Suite 330,          
  Boston, MA 02111-1307, USA.                                           
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>

#include <gtk/gtk.h>

/* global stuff */
#include "interface.h"
#include "support.h"
#include "callbacksproto.h"

/* private stuff */
#include "supportpriv.h"
#include "lineakconfig.h"

//#include <dmalloc.h>

/**********************************************************************************************/

/* refresh *ALL* the data on the GUI (this is called on startup) */
gint refreshGUIdata(GtkWidget *widget) {
  extern gboolean changing_text_from_signal;
  GtkWidget *branditem = NULL;
  
  changing_text_from_signal = TRUE;
  /* refresh EAK types */
  branditem = refresh_kbtypes (widget);
  
  /* refresh CD-ROM entry */
  refresh_cdromdev (widget);
  
  /* refresh CD-ROM entry */
  refresh_mixdev (widget);

  /* refresh keyboard image */
  refresh_kbimage (widget);

  /* refresh key list */
  refresh_keylist (widget);
  
  /* refresh command/action */
  refresh_specials (widget);
  refresh_keycommand (widget, 0);
  
  /* make sure the brands refresh on startup */
  if (branditem!=NULL) gtk_menu_item_activate(GTK_MENU_ITEM(branditem));

  /* done */
  changing_text_from_signal = FALSE;
  return (TRUE);
}

/* only needed at startup */
GtkWidget *refresh_kbtypes (GtkWidget *widget) {
  GtkWidget *guiwidget, *menuwidget, *menuitem, *firstitem=NULL, *actitem=NULL;
  abrand_struct *brandptr=availbrands;
  gint i=0, index=0;
  /* just brands, models go by event */
  guiwidget = lookup_widget(widget, "kbbrand");
  gtk_option_menu_remove_menu(GTK_OPTION_MENU (guiwidget));
  menuwidget = gtk_menu_new();
  while (brandptr!=NULL) {
    menuitem = add_kbbrandmenu_item(menuwidget, brandptr->name, TRUE);
	if (firstitem==NULL) firstitem=menuitem;
    if (myEAK.EAKbrand!=NULL && (strcasecmp(brandptr->name, myEAK.EAKbrand)==0)) {
	  index=i;
      actitem = menuitem;
	}
	i++;
	brandptr=brandptr->next;
  }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (guiwidget), menuwidget);
  if (actitem!=NULL) {
    gtk_option_menu_set_history(GTK_OPTION_MENU (guiwidget), index);
	return actitem;
  } else {
    gtk_option_menu_set_history(GTK_OPTION_MENU (guiwidget), 0);
	return firstitem;
  }
}

/* only needed at startup */
gint refresh_cdromdev (GtkWidget *widget) {
  GtkWidget *guiwidget;
  guiwidget = lookup_widget(widget, "cdromentry");
  gtk_entry_set_text(GTK_ENTRY (guiwidget), (gchar *)cdromdev);
  return (TRUE);  
}
/* only needed at startup */
gint refresh_mixdev (GtkWidget *widget) {
  GtkWidget *guiwidget;
  guiwidget = lookup_widget(widget, "mixerentry");
  gtk_entry_set_text(GTK_ENTRY (guiwidget), (gchar *)mixdev);
  return (TRUE);  
}

/* ******** */

GtkWidget *refresh_kbmodels (GtkWidget *widget, char *brandname) {
  GtkWidget *guiwidget, *menuwidget, *menuitem, *firstitem=NULL, *actitem=NULL;
  dcfp_section_struct *sectptr=NULL;
  const char *brand=NULL, *model=NULL;
  int i=0, index=0;
  /* update models for specific brand */
  if (brandname == NULL) return FALSE;
  guiwidget = lookup_widget(widget, "kbmodel");
  gtk_option_menu_remove_menu(GTK_OPTION_MENU (guiwidget));
  menuwidget = gtk_menu_new();
  sectptr = defdata->sections;
  while (sectptr != NULL) {
	brand = dcfp_get_var_from_section(sectptr, "brandname");
	if (brand!=NULL && (strcasecmp(brand, brandname)==0)) {
  	  model = dcfp_get_var_from_section(sectptr, "modelname");
      menuitem = add_kbmodelmenu_item(menuwidget, model, TRUE);
	  if (firstitem==NULL) firstitem=menuitem;
      if (myEAK.EAKmodel!=NULL && (strcasecmp(model, myEAK.EAKmodel)==0)) {
	    index=i;
		actitem = menuitem;
	  }
	  i++;
	}
	sectptr = sectptr->next;
  }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (guiwidget), menuwidget);
  if (actitem!=NULL) {
    gtk_option_menu_set_history(GTK_OPTION_MENU (guiwidget), index);
	gtk_menu_item_activate(GTK_MENU_ITEM(actitem));
  } else {
    gtk_option_menu_set_history(GTK_OPTION_MENU (guiwidget), 0);
    gtk_menu_item_activate(GTK_MENU_ITEM(firstitem));
  }	
  return actitem;
}

gint refresh_kbimage (GtkWidget *widget) {
  GtkWidget *kbimage;
  char *pixmapfile;
  kbimage = lookup_widget(widget, "kbimage");
  gtk_widget_hide(kbimage);
  pixmapfile = malloc( (strlen(myEAK.EAKtype) + 5) * sizeof(char));
  strcpy(pixmapfile, myEAK.EAKtype);
  strcat(pixmapfile, ".xpm\0");
  replace_my_pixmap_from_file(widget, kbimage,pixmapfile);
  gtk_widget_show(kbimage);
  free(pixmapfile);
  return (TRUE);
}

/* refresh the keylist when the user selected some other keyboard model */
gint refresh_keylist (GtkWidget *widget) {
  GtkWidget *guiwidget;
  EAkey *thiskey=NULL;
  guiwidget = lookup_widget(widget, "keylist");
  gtk_list_clear_items(GTK_LIST (guiwidget), 0, -1);
  for (thiskey=myEAK.EAKeylist; thiskey!=NULL; thiskey=thiskey->next)
	add_list_item(guiwidget, thiskey->EAkeyname);
  gtk_signal_connect (GTK_OBJECT (guiwidget), "unselect-child", GTK_SIGNAL_FUNC (on_keylist_unselect), NULL);
  gtk_list_select_item(GTK_LIST (guiwidget), 0);
  selected_key = 0;
  return (TRUE);  
}

/* refresh the command bound to a key when it's selected */
gint refresh_keycommand (GtkWidget *widget, gint keyindex) {
  extern gboolean changing_text_from_signal;
  GtkWidget *guiwidget;
  guint i=0;
  EAkey *thiskey;
  changing_text_from_signal = TRUE;
  for (i=0, thiskey=myEAK.EAKeylist; i!=keyindex && thiskey!=NULL; i++, thiskey=thiskey->next);
  if (thiskey==NULL) {
	g_warning("invalid key index: %d\n",keyindex);
	return (FALSE);
  }

  if (thiskey->EAcommand!=NULL && is_special_action((gchar *)(thiskey->EAcommand))) { /* special action */
	guiwidget = lookup_widget(widget, "radiocommand");
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (guiwidget), FALSE);
	gtk_widget_hide(guiwidget);
	GTK_TOGGLE_BUTTON(guiwidget)->active = FALSE;
	gtk_widget_show(guiwidget);
	guiwidget = lookup_widget(widget, "radiospecial");
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (guiwidget), TRUE);
	gtk_widget_hide(guiwidget);
	GTK_TOGGLE_BUTTON(guiwidget)->active = TRUE;
	gtk_widget_show(guiwidget);
	guiwidget = lookup_widget(widget, "commandentry");
	gtk_entry_set_text(GTK_ENTRY (guiwidget), "");
	gtk_entry_set_editable(GTK_ENTRY(guiwidget), FALSE);
	for (i=0; strcmp(specialacts[i].action,thiskey->EAcommand)!=0 && i<NR_SPECIALS; i++);
    guiwidget = lookup_widget(widget, "specials");
	gtk_option_menu_set_history(GTK_OPTION_MENU (guiwidget), i);
  } else {                                                        /* system command */
	guiwidget = lookup_widget(widget, "radiospecial");
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (guiwidget), FALSE);
	gtk_widget_hide(guiwidget);
	GTK_TOGGLE_BUTTON(guiwidget)->active = FALSE;
	gtk_widget_show(guiwidget);
	guiwidget = lookup_widget(widget, "radiocommand");
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (guiwidget), TRUE);
	gtk_widget_hide(guiwidget);
	GTK_TOGGLE_BUTTON(guiwidget)->active = TRUE;
	gtk_widget_show(guiwidget);
	guiwidget = lookup_widget(widget, "commandentry");
	gtk_entry_set_text(GTK_ENTRY (guiwidget), thiskey->EAcommand!=NULL?(gchar *)(thiskey->EAcommand):"");
	gtk_entry_set_editable(GTK_ENTRY(guiwidget), TRUE);
  }
  changing_text_from_signal = FALSE;
  return (TRUE);  
}

/* refill special action menu (only needed on startup) */
gint refresh_specials (GtkWidget *widget) {
  GtkWidget *guiwidget, *menuwidget;
  gint i=0;
  guiwidget = lookup_widget(widget, "specials");
  gtk_option_menu_remove_menu(GTK_OPTION_MENU (guiwidget));
  menuwidget = gtk_menu_new();
  for (i=0; i<NR_SPECIALS; i++)
    add_specmenu_item(menuwidget, specialacts[i].action, specialacts[i].longname, TRUE);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (guiwidget), menuwidget);
  return (TRUE);
}

/* convenient for adding new menu items, return the item only if it is the same as myEAK.EAKbrand */
GtkWidget *add_kbbrandmenu_item (GtkWidget *menu, gchar *name, gint add_callback) {
  GtkWidget *item;
  item = gtk_menu_item_new_with_label (name);
  gtk_menu_append (GTK_MENU (menu), item);
  if (add_callback)
	gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (on_kbbrandmenuitem_activate), name);
  gtk_widget_show (item);
  return item;
}

/* convenient for adding new menu items */
GtkWidget *add_kbmodelmenu_item (GtkWidget *menu, const char *name, gint add_callback) {
  GtkWidget *item;
  item = gtk_menu_item_new_with_label (name);
  gtk_menu_append (GTK_MENU (menu), item);
  if (add_callback)
	gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (on_kbmodelmenuitem_activate), (char *)name);
  gtk_widget_show (item);
  return item;
}

/* convenient for adding new menu items */
GtkWidget *add_specmenu_item (GtkWidget *menu, char *act, gchar *name, gint add_callback) {
  GtkWidget *item;
  item = gtk_menu_item_new_with_label (name);
  gtk_menu_append (GTK_MENU (menu), item);
  if (add_callback)
	gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (on_specmenuitem_activate), act);
  gtk_widget_show (item);
  return item;
}

/* convenient for adding new list items */
GtkWidget *add_list_item (GtkWidget *list, gchar *name) {
  GtkWidget *item;
  item = gtk_list_item_new_with_label (name);
  gtk_container_add(GTK_CONTAINER (list), item);
  gtk_signal_connect (GTK_OBJECT (item), "select", GTK_SIGNAL_FUNC (on_keylist_select), name);
  gtk_widget_show (item);
  return item;
}

/* is a command a special action? */
gint is_special_action (gchar *command) {
  if (strcmp(command, EAK_EJECT)==0 ||
	  strcmp(command, EAK_VOLUP)==0 ||
      strcmp(command, EAK_VOLDOWN)==0 ||
      strcmp(command, EAK_MUTE)==0 ||
      strcmp(command, EAK_SLEEP)==0)
    return (TRUE);
  else
	return (FALSE);
}

/* select a new keyboard brand (called from callback) */
void select_new_kbbrand(GtkWidget *widget, gpointer data) {
  GtkWidget *guiwidget;
  if (!inNCwindow) {
    /* clear keylist */
    guiwidget = lookup_widget(widget, "keylist");
    gtk_list_clear_items(GTK_LIST (guiwidget), 0, -1);
	/* clear command entry box */
    guiwidget = lookup_widget(widget, "commandentry");
	gtk_entry_set_text(GTK_ENTRY (guiwidget), "");
	//gtk_entry_set_editable(GTK_ENTRY(guiwidget), FALSE);
  }
  myEAK.EAKbrand = (char *)data;
  /* update models */
  refresh_kbmodels(widget, (char *)data);
}

/* select a new keyboard model (called from callback) */
void select_new_kbmodel(GtkWidget *widget, gpointer data) {
  extern gboolean changing_text_from_signal;
  char *temp;
  myEAK.EAKmodel = (char *)data;
  
  if (kbtype!=NULL) free(kbtype);
  
  if ((temp = lookup_kbtype(myEAK.EAKbrand, myEAK.EAKmodel))==NULL) {
	g_print("*** PANIC: keyboard of brand '%s' and model '%s' not found!\n",myEAK.EAKbrand, myEAK.EAKmodel);
	cleanexit();
  } else {
	kbtype = malloc((strlen(temp)+1)*sizeof(char));
	sprintf(kbtype,"%s",temp);
  }
  cleanKeylist();
  initEAK();
  if (!inNCwindow) {
    changing_text_from_signal = TRUE;
    refresh_keylist (widget);
    refresh_keycommand (widget, 0); // refresh_keylist selects 1st key, hence index 0
    refresh_kbimage (widget);
    changing_text_from_signal = FALSE;
    selected_key = 0;
  }
}

/* lookup a kbtype by brand/model from def data */
char *lookup_kbtype(char *brand, char *model) {
  dcfp_section_struct *sectptr=NULL;
  const char *brandname=NULL, *modelname=NULL;

  sectptr = defdata->sections;
  while (sectptr != NULL) {
	brandname = dcfp_get_var_from_section(sectptr, "brandname");
	if (brandname!=NULL && (strcasecmp(brand, brandname)==0)) {
  	  modelname = dcfp_get_var_from_section(sectptr, "modelname");
  	  if (modelname!=NULL && (strcasecmp(model, modelname)==0)) {
		return sectptr->name;
	  }
	}
	sectptr = sectptr->next;
  }
  return NULL;
}

/* change the key command data to the users input */
void change_key_command(EAkey *thiskey, gchar *entrytext) {
  if (thiskey->EAcommand!=NULL) free(thiskey->EAcommand);
  thiskey->EAcommand = malloc((strlen(entrytext)+1)*sizeof(char));
  sprintf(thiskey->EAcommand, "%s", entrytext);
}

/* wrapper for change_key_command, lookups the selected key */
void change_selectedkey_command(gchar *entrytext) {
  EAkey *thiskey;
  guint i;
  for (i=0, thiskey=myEAK.EAKeylist; i!=selected_key && thiskey!=NULL; i++, thiskey=thiskey->next);
  if (thiskey==NULL) {
    g_warning("invalid selected key index: %d\n",selected_key);
    g_free(entrytext);
    return;
  }
  if (strcmp(thiskey->EAcommand!=NULL?thiskey->EAcommand:"",(char *)entrytext) != 0)
	change_key_command(thiskey, entrytext);
}

/**********************************************************************************************/

/* for looking up a widget */
GtkWidget *lookup_widget (GtkWidget *widget, const gchar *widget_name) {
  GtkWidget *parent, *found_widget;
  for (;;) {
    if (GTK_IS_MENU (widget))
      parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
    else
      parent = widget->parent;
    if (parent == NULL)
      break;
    widget = parent;
  }
  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget), widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

/* This is an internally used function to create pixmaps. */
GtkWidget* create_my_pixmap (GtkWidget *widget, char **thepixmap) {
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask,
                                                     NULL, thepixmap);
  if (gdkpixmap == NULL)
    g_error ("Couldn't create pixmap.");
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

/* This is an internally used function to create pixmaps. */
GtkWidget *create_my_pixmap_from_file (GtkWidget *widget, const gchar *filename) {
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;

  if (!filename || !filename[0])
      return create_my_pixmap (widget, dummy_pixmap_xpm);

  found_filename = check_file_exists (PACKAGE_PIXMAPS_DIR, filename);

  /* If we haven't found the pixmap, try the source directory (so I see a pixmap while developing ;-)) */
  if (!found_filename) {
    found_filename = check_file_exists ("../pixmaps", filename);
  }
  if (!found_filename) {
    found_filename = check_file_exists ("pixmaps", filename);
  }

  /* still not found... warning and a dummy pixmap */
  if (!found_filename) {
    g_warning (_("Couldn't find pixmap file: %s"), filename);
    return create_my_pixmap (widget, dummy_pixmap_xpm);
  }

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, found_filename);
  if (gdkpixmap == NULL) {
    g_warning (_("Error loading pixmap file: %s"), found_filename);
    g_free (found_filename);
    return create_my_pixmap (widget, dummy_pixmap_xpm);
  }
  g_free (found_filename);
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

void replace_my_pixmap_from_file (GtkWidget *widget, GtkWidget *thepixmap, const gchar *filename) {
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap=NULL;
  GdkBitmap *mask;
  gboolean noimage = FALSE;

  if (!filename || !filename[0]) {
	noimage = TRUE;
  } else {

    found_filename = check_file_exists (PACKAGE_PIXMAPS_DIR, filename);

    /* If we haven't found the pixmap, try the source directory (so I see a pixmap while developing ;-)) */
    if (!found_filename) {
      found_filename = check_file_exists ("../pixmaps", filename);
    }
    if (!found_filename) {
      found_filename = check_file_exists ("pixmaps", filename);
    }

    /* still not found... warning and a dummy pixmap */
    if (!found_filename) {
	  noimage = TRUE;
    }
  }
 
  colormap = gtk_widget_get_colormap (widget);
  if (noimage) {
    found_filename = check_file_exists (PACKAGE_PIXMAPS_DIR, NOIMAGEXPM);
    if (!found_filename)
      found_filename = check_file_exists ("../pixmaps", NOIMAGEXPM);
    if (!found_filename)
      found_filename = check_file_exists ("pixmaps", NOIMAGEXPM);
  }
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask, NULL, found_filename);
  if (gdkpixmap == NULL) {
    g_warning (_("Error reloading pixmap file: %s"), found_filename);
    g_free (found_filename);
    return;
  }
  g_free (found_filename);
  gtk_pixmap_set (GTK_PIXMAP(thepixmap), gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return;
}

/* This is an internally used function to check if a pixmap file exists. */
gchar *check_file_exists (const gchar *directory, const gchar *filename) {
  gchar *full_filename;
  struct stat s;
  gint status;

  full_filename = (gchar*) g_malloc (strlen (directory) + 1
                                     + strlen (filename) + 1);
  strcpy (full_filename, directory);
  strcat (full_filename, G_DIR_SEPARATOR_S);
  strcat (full_filename, filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;
  g_free (full_filename);
  return NULL;
}

/**********************************************************************************************/

/* parse .conf file */
gint parseconffile(void) {
  FILE *fp = NULL;
  gchar *homedir = NULL;
  GtkWidget *awindow, *branditem=NULL;
  char *temp;
  
  if (cfile==NULL) {
    homedir = (gchar *) getenv("HOME");
    cfile = g_malloc((strlen(homedir) + strlen(CONFFILE) + 1) * sizeof(gchar));
	sprintf(cfile, "%s%s", homedir, CONFFILE);
  }

  /* does the file exist, if not give user opportunity to generate one */
  if ((fp = fopen(cfile, "r")) == NULL) {
	inNCwindow = TRUE;
    awindow = create_NewConfWindow("No configuration file found, please choose your keyboard and generate one");
    branditem=refresh_kbtypes (awindow);
    if (branditem!=NULL) gtk_menu_item_activate(GTK_MENU_ITEM(branditem));
	gtk_widget_show(awindow);
	gtk_main();
	gtk_widget_destroy(awindow);
	inNCwindow = FALSE;
    FirstTimeBrand=TRUE; FirstTimeModel=TRUE;
  } else {
	fclose (fp);
  }

  /* does the file exist? */
  if ((fp = fopen(cfile, "r")) != NULL) {
	fclose(fp);
	/* backup filename */
	backupfile = g_malloc((strlen(homedir) + strlen(BAKFILE) + 1) * sizeof(gchar));
	sprintf(backupfile, "%s%s", homedir, BAKFILE);
	/* parse the contents of the conf file */
	if ( (confdata = dcfp_parse_file(cfile)) == NULL ) {
	  /* error parsing file */
	  g_print("FATAL ERROR occurred while reading configuration data!\n");
	  return (FALSE);
	}
	/* store keyboard type globally */
    if ((temp = (char *)dcfp_get_global_var(confdata, "KeyboardType"))==NULL) {
	  g_print ("*** Error in config file: KeyboardType not found!\n");
	  cleanexit();
    } else {
	  if (kbtype!=NULL) { free(kbtype); kbtype = NULL; }
	  kbtype = malloc ((strlen(temp)+1)*sizeof(char));
	  sprintf(kbtype,"%s",temp);
	}
    /* store cdrom device globally, defaults to /dev/cdrom */
    if ((temp = (char *)dcfp_get_global_var(confdata, "CdromDevice"))==NULL) {
	  g_print ("*** CdromDevice not found in config file, setting default to /dev/cdrom...\n");
	  if (cdromdev !=NULL) { free(cdromdev); cdromdev = NULL; }
	  cdromdev = (char *)malloc(11*sizeof(char));
	  sprintf(cdromdev, "%s", "/dev/cdrom");
    } else {
	  if (cdromdev !=NULL) { free(cdromdev); cdromdev = NULL; }
	  cdromdev = (char *)malloc((strlen(temp)+1)*sizeof(char));
	  sprintf(cdromdev, "%s", temp);
	}
    /* store mixer device globally, defaults to /dev/mixer */
    if ((temp = (char *)dcfp_get_global_var(confdata, "MixerDevice"))==NULL) {
	  g_print ("*** MixerDevice not found in config file, setting default to /dev/mixer...\n");
	  if (mixdev!=NULL) { free(mixdev); mixdev = NULL; }
	  mixdev = (char *)malloc(11*sizeof(char));
	  sprintf(mixdev, "%s", "/dev/mixer");
    } else {
	  if (mixdev!=NULL) { free(mixdev); mixdev = NULL; }
	  mixdev = (char *)malloc((strlen(temp)+1)*sizeof(char));
	  sprintf(mixdev, "%s", temp);
	}
  } else {
	/* file does not exist, user should have created one... giving up */
    g_print("FATAL ERROR: configuration file %s not found!\n             use 'lineakd -c TYPE' to create one\n",cfile);
  	return (FALSE);
  }
  
  return (TRUE);
}

/* parse keyboard definition file */
gint parsedeffile(void) {
  FILE *fp = NULL, *fu;
  char *home;
  dcfp_section_struct *sectptr=NULL;
  dcfp_var_struct *varptr=NULL;
  /* SET full path of definition file ($sysconfdir/lineakkb.def) */
  home = getenv("HOME");
  if (usrdfile==NULL) {
	usrdfile = malloc( (strlen(home) + 1 + 7 + 1 + strlen(DEFFILE) +1) * sizeof(char));
	sprintf(usrdfile,"%s/.lineak/%s", home, DEFFILE);
	if ((fu=fopen(usrdfile,"r"))!=NULL) {
	  fclose(fu);
	  /* it exists, parse the contents */
	  if ( (usrdefdata = dcfp_parse_file(usrdfile)) == NULL ) {
	    /* error parsing file */
	    g_print("FATAL ERROR occurred while reading definition data!\n");
  	    return(FALSE);
	  }
	}
  }	
  if (dfile == NULL) {
	dfile = malloc( (strlen(CONFDIR) + 1 + strlen(DEFFILE) +1) * sizeof(char));
	sprintf(dfile,"%s/%s", CONFDIR, DEFFILE);
  }
  
  /* does the file exist? */
  if ((fp = fopen(dfile, "r")) != NULL) {
    fclose(fp);
	/* it exists, parse the contents */
	if ( (defdata = dcfp_parse_file(dfile)) == NULL ) {
	  /* error parsing file */
	  g_print("FATAL ERROR occurred while reading definition data!\n");
  	  return(FALSE);
	}
	if (usrdefdata !=NULL) {
      varptr = defdata->globalvars;
	  if (varptr==NULL) {
	    ((dcfp_data_struct *)defdata)->globalvars = usrdefdata->globalvars;
	  } else {
	    while (varptr->next != NULL) varptr = varptr->next;
	    varptr->next = usrdefdata->globalvars;
	  }
      sectptr = defdata->sections;
	  if (sectptr == NULL) {
	    ((dcfp_data_struct *)defdata)->sections = usrdefdata->sections;
	  } else {
  	    while (sectptr->next != NULL) sectptr = sectptr->next;
	    sectptr->next = usrdefdata->sections;
	  }
	}
	availbrands = get_available_brands_from_data(defdata);
  } else {
	/* file does not exist */
    g_print("FATAL ERROR: definition file %s not found!\n",dfile);
	return(FALSE);
  }

  return (TRUE);
}

/* get all the different brands from def data */
abrand_struct *get_available_brands_from_data(const dcfp_data_struct *data) {
  abrand_struct *brands=NULL, *tmp=NULL;
  dcfp_section_struct *sectptr=NULL;
  char *brandname=NULL;
  const char *tmpvar=NULL;
  int firstbrand=1;
  if (data == NULL) return NULL;
  sectptr = data->sections;
  while (sectptr != NULL) {
	if ((tmpvar = dcfp_get_var_from_section(sectptr, "brandname"))==NULL) break; //TODO: ERROR
	brandname = malloc((strlen(tmpvar)+1)*sizeof(char));
	sprintf (brandname, "%s", tmpvar);
	if (brandname!=NULL && !brand_exists(brandname, brands)) {
	  if (firstbrand) {
		firstbrand = 0;
		tmp = brands = (abrand_struct *)malloc(sizeof(abrand_struct));
	  } else {
		tmp->next = (abrand_struct *)malloc(sizeof(abrand_struct));
		tmp = tmp->next;
	  }
	  tmp->name = brandname;
	  tmp->next = NULL;
	}
	brandname = NULL;
	sectptr = sectptr->next;
  }
  return brands;
}

void destroy_brand_tree(abrand_struct *brandtree) {
  abrand_struct *tmp, *tmpnext;
  tmp = brandtree;
  while (tmp !=NULL) {
	tmpnext = tmp->next;
	if(tmp->name!=NULL) free(tmp->name);
	free(tmp);
	tmp = tmpnext;
  }
}

/* check if a brand already exists... */
gboolean brand_exists(char *brand, abrand_struct *brands) {
  abrand_struct *tmp;
  if (brand==NULL) return FALSE;
  tmp = brands;
  while (tmp !=NULL) {
	if (strcasecmp(brand, tmp->name) == 0) return TRUE;
	tmp = tmp->next;
  }
  return FALSE;
}

/**********************************************************************************************/

/* cleanup */
gint cleanexit() {
  //destroy_brand_tree(availbrands);//memory bug
  free(myEAK.EAKbrand); myEAK.EAKbrand=NULL;
  free(myEAK.EAKmodel); myEAK.EAKmodel=NULL;
  //cleanKeylist();
  dcfp_destroy_data_struct((dcfp_data_struct *)confdata);
  dcfp_destroy_data_struct((dcfp_data_struct *)defdata);
  if (usrdefdata!=NULL) free((dcfp_data_struct *)usrdefdata);
  if (kbtype!=NULL) { free(kbtype); kbtype=NULL; }
  if (cdromdev!=NULL) { free(cdromdev); cdromdev = NULL; }
  if (mixdev!=NULL) { free(mixdev); mixdev = NULL; }
  free(cfile);
  free(dfile);
  gtk_main_quit();
  return FALSE;
}

/**********************************************************************************************/

/* init the EAKeyboard */
gint initEAK (void) {
  const dcfp_section_struct *kbsect=NULL, *keysect=NULL;
  dcfp_var_struct *keyptr=NULL;
  EAkey *tmp = NULL;
  const char *tmpvar=NULL;
  int keycount = 0;

  if ((kbsect = dcfp_get_global_section_by_name(defdata, kbtype)) == NULL) {
	printf ("\n*** ERROR: your keyboard (%s) was not found in definition files \n\n",kbtype);
	cleanexit();
  }
  myEAK.EAKtype = kbtype;
  if ((tmpvar = dcfp_get_var_from_section(kbsect, "brandname"))==NULL) {
	printf ("\n*** ERROR: no 'brandname' found in the section of your keyboard (%s)\n\n",kbtype);
	cleanexit();
  } else {
	if (myEAK.EAKbrand!=NULL) free(myEAK.EAKbrand);
    myEAK.EAKbrand = malloc((strlen(tmpvar)+1)*sizeof(char));
	sprintf(myEAK.EAKbrand, "%s", tmpvar);
  }
  if ((tmpvar = dcfp_get_var_from_section(kbsect, "modelname"))==NULL) {
	printf ("\n*** ERROR: no 'modelname' found in the section of your keyboard (%s)\n\n",kbtype);
	cleanexit();
  } else {
	if (myEAK.EAKmodel!=NULL) free(myEAK.EAKmodel);
    myEAK.EAKmodel = malloc((strlen(tmpvar)+1)*sizeof(char));
	sprintf(myEAK.EAKmodel, "%s", tmpvar);
  }
  if ((keysect = dcfp_get_subsection_by_name(kbsect, "KEYS")) == NULL) {
	printf("\n*** Error: no [KEYS] section found in your keyboard definition (%s)\n\n",kbtype);
	cleanexit();
  }
  if ((keyptr = keysect->globalvars) == NULL) {
	printf("\n*** Error: no keys found in the [KEYS] section of your keyboard (%s)\n\n",kbtype);
	cleanexit();
  }
  if (keyptr != NULL) {
	myEAK.EAKeylist = tmp = malloc(sizeof(EAkey));
	while(keyptr!=NULL) {
      keycount++;
	  tmp->EAcommand = NULL;
	  if ( keyptr->key==NULL || *(keyptr->key)=='\0') {
		printf("*** ERROR while fetching keyname for key with keycode %d from configuration file!\n",tmp->EAkeycode);
		cleanexit();
	  } else {
		tmp->EAkeyname = malloc((strlen(keyptr->key)+1)*sizeof(char));
		sprintf(tmp->EAkeyname,"%s",keyptr->key);
	  }
	  if ( (tmpvar = dcfp_get_global_var(confdata, tmp->EAkeyname))==NULL) {
		//printf("*** ERROR while fetching command for key '%s' from configuration file!\n",tmp->EAkeyname);
		tmp->EAcommand = malloc(sizeof(char));
		*(tmp->EAcommand) = '\0';
	  } else {
		tmp->EAcommand = malloc((strlen(tmpvar)+1)*sizeof(char));
		sprintf(tmp->EAcommand,"%s",tmpvar);
	  }
	  if (keyptr->next != NULL) {
		tmp->next = malloc(sizeof(EAkey));
		tmp = tmp->next;
	  } else {
		tmp->next = NULL;
	  }
	  keyptr = keyptr->next;
	}
  } else {
	myEAK.EAKeylist = NULL;
  }
  return TRUE;
}

/* destroy the EAkeys linked list */
void cleanKeylist (void) {
  EAkey *tmp = NULL, *tmp2 = NULL;
  tmp = myEAK.EAKeylist;
  while (tmp!=NULL) {
	tmp2 = tmp->next;
	if (tmp->EAkeyname!=NULL) free(tmp->EAkeyname);
	if (tmp->EAcommand!=NULL) free(tmp->EAcommand);
	free(tmp);
	tmp = tmp2;
  }
}

/**********************************************************************************************/

/* backup the conf file so we can cancel the changes */
gboolean backup_conffile (void) {
  FILE *orig, *backup;
  int chr;
  if ((orig=fopen(cfile,"r"))==NULL) {
	g_warning("error opening configuration file %s for read\n",cfile);
	return FALSE;
  }
  if ((backup=fopen(backupfile, "w"))==NULL) {
	g_warning("error opening backup configuration file %s for write\n",backupfile);
	fclose(orig);
	return FALSE;
  }
  while ((chr=fgetc(orig))!=EOF) fputc(chr,backup);
  fclose(orig);
  fclose(backup);
  return TRUE;
}

/* restore the conf file from backup */
gboolean restore_conffile (void) {
  FILE *orig, *dest;
  int chr;
  if ((orig=fopen(backupfile,"r"))==NULL) {
	g_warning("error opening backup configuration file %s for read\n",backupfile);
	return FALSE;
  }
  if ((dest=fopen(cfile, "w"))==NULL) {
	g_warning("error opening configuration file %s for write\n",cfile);
	fclose(orig);
	return FALSE;
  }
  while ((chr=fgetc(orig))!=EOF) fputc(chr,dest);
  fclose(orig);
  fclose(dest);
  return TRUE;
}

/* save the settings to the conf file */
gboolean save_conffile (void) {
  FILE *dest;
  EAkey *key;
  char *lineakdir, *homedir;
  
  /* check if $HOME/.lineak/ directory exists, if not create it */
  homedir = getenv("HOME");
  lineakdir = malloc( (strlen(homedir) + 1 + 7 + 1) * sizeof(char));
  sprintf(lineakdir, "%s/.lineak", homedir);
  if (opendir(lineakdir) == NULL) {
    if (mkdir(lineakdir, 0755) == -1) {
      printf("*** FATAL ERROR: unable to create directory %s !\n",lineakdir);
	  cleanexit();
    } 
  }
  free(lineakdir);
  /* open file for write */
  if ((dest=fopen(cfile, "w"))==NULL) {
	g_warning("error opening configuration file %s for write\n",cfile);
	return FALSE;
  }
  fprintf(dest,"%s# LinEAK Configuration for %s %s\n# (generated by lineakconfig)\n\n# Global settings\n"
               "KeyboardType\t\t= %s\n"
               "CdromDevice\t\t= %s\n"
               "MixerDevice\t\t= %s\n"
               "\n# Specific keys of your keyboard\n", CONF_HEADER, myEAK.EAKbrand, myEAK.EAKmodel, kbtype, cdromdev!=NULL?cdromdev:"", mixdev!=NULL?mixdev:"");
  key = myEAK.EAKeylist;
  while (key!=NULL) {
	fprintf(dest,"%s%s= \"%s\"\n",key->EAkeyname,strlen(key->EAkeyname)>7?"\t":"\t\t",key->EAcommand!=NULL?key->EAcommand:"");
	key = key->next;
  }
  fprintf(dest,"\n# end lineakd.conf\n");
  fclose(dest);
  return TRUE;
}

/* check if a file exists */
int file_exists(char *filename) {
  FILE *fp;
  if ((fp = fopen(filename, "r"))!=NULL) {
	fclose (fp);
	return 1;
  } else
	return 0;
}

/* for deleting a file */
void delete_file(char *filename) {
  char *rmfile=NULL;
  if (filename == NULL) return;
  if (file_exists(filename)) {
    rmfile = malloc ( ( 3 + strlen(filename) + 17 + 1) * sizeof(char));
    sprintf(rmfile, "rm %s >>/dev/null 2>&1", filename);
    system(rmfile);
    free(rmfile);
  }
}

gint get_lineakd_pid(void) {
  FILE *fp;
  int pspid=0;
  char *home, pidstr[9], *pidtestcmd=NULL, *pidtestfile=NULL;
  pidstr[0]='\0';
  
  home = getenv("HOME");
  pidtestfile = malloc ( ( strlen(home) + strlen(PIDFILE) + 7) * sizeof(char));
  sprintf(pidtestfile, "%s%s.ctemp", home, PIDFILE);

  /* get pid from 'ps' and store in pidtestfile */
  pidtestcmd = malloc( ( 76 + strlen(pidtestfile) + 1) * sizeof(char));
  sprintf(pidtestcmd, "ps -ef | grep \"lineakd \" | grep $USER | grep -v grep | awk '{ print $2 }' > %s", pidtestfile);
  system(pidtestcmd);
  /* get pid from pidtestfile */
  if ((fp = fopen(pidtestfile, "r"))!=NULL) {
    if (fgets(pidstr, 8, fp) != NULL) {
	  if (pidstr[strlen(pidstr)-1] == '\n' )
		pidstr[strlen(pidstr)-1] = '\0';
	  sscanf(pidstr, "%d", &pspid);
	}
	fclose(fp);
    delete_file(pidtestfile);
  } else {
	printf("*** Error: seem to have lost PID TEST file. ~ I'm confused ~ ...\n");
  }
  free(pidtestfile);
  return pspid;
}


/**********************************************************************************************/
