/*
  LinEAK - Linux support for Easy Access and Internet Keyboards      
  Copyright (C) 2001,2002 Mark Smulders <Mark@PIRnet.nl>
  
  lineakconfig, GTK+ interface for configuring lineakd

  This program is free software; you can redistribute it and/or modify  
  it under the terms of the GNU General Public License as published by  
  the Free Software Foundation; either version 2, or (at your option)   
  any later version.                                                    
                                                                        
  This program is distributed in the hope that it will be useful,       
  but WITHOUT ANY WARRANTY; without even the implied warranty of        
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
  GNU General Public License for more details.                          
                                                                        
  You should have received a copy of the GNU General Public License     
  along with this program; if not, write to the                         
  Free Software Foundation, Inc., 59 Temple Place - Suite 330,          
  Boston, MA 02111-1307, USA.                                           
*/

#ifndef HAVE_SUPPORT_H
#define HAVE_SUPPORT_H

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <X11/Xlib.h>
#include <X11/XKBlib.h>
#include <X11/extensions/XKBfile.h>

#include "dcfp.h"
#include "lineakcommon.h"

/**********************************************************************************************/

/* Standard gettext macros. */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

/**********************************************************************************************/

/* the keyboard */
EAKeyboard myEAK;


/* function prototypes */
gint parseconffile(void);
gint parsedeffile(void);
gint initEAK (void);
void cleanKeylist (void);
gint cleanexit(void);
gint refreshGUIdata(GtkWidget *widget);
GtkWidget *refresh_kbtypes (GtkWidget *widget);
gint refresh_cdromdev (GtkWidget *widget);
gint refresh_mixdev (GtkWidget *widget);
GtkWidget *refresh_kbmodels (GtkWidget *widget, char *brandname);
gint refresh_kbimage (GtkWidget *widget);
gint refresh_keylist (GtkWidget *widget);
gint refresh_keycommand (GtkWidget *widget, gint keyindex);
gint refresh_specials (GtkWidget *widget);
void select_new_kbbrand(GtkWidget *widget, gpointer data);
void select_new_kbmodel(GtkWidget *widget, gpointer data);
char *lookup_kbtype(char *brand, char *model);
void change_key_command(EAkey *thiskey, gchar *entrytext) ;
void change_selectedkey_command(gchar *entrytext);
gboolean backup_conffile (void);
gboolean restore_conffile (void);
gboolean save_conffile (void);
int file_exists(char *filename);
void delete_file(char *filename);
gboolean get_lineakd_pid(void);

/* available brands */
typedef struct abrands {
  char *name;
  struct abrands *next;
} abrand_struct;
abrand_struct *get_available_brands_from_data(const dcfp_data_struct *data);
void destroy_brand_tree(abrand_struct *brandtree);
gboolean brand_exists(char *brand, abrand_struct *brands);

/* various init */
char *kbtype, *cdromdev, *mixdev;
gint selected_key;
const dcfp_data_struct *defdata, *usrdefdata, *confdata;
abrand_struct *availbrands;
gboolean FirstTimeBrand, FirstTimeModel;

/*
 * This function returns a widget in a component created by Glade.
 * Call it with the toplevel widget in the component (i.e. a window/dialog),
 * or alternatively any widget in the component, and the name of the widget
 * you want returned.
 */
GtkWidget*  lookup_widget (GtkWidget *widget, const gchar *widget_name);

/* get_widget() is deprecated. Use lookup_widget instead. */
#define get_widget lookup_widget

/* pixmap related */
GtkWidget* create_my_pixmap (GtkWidget *widget, char **thepixmap);
gchar *check_file_exists (const gchar *directory, const gchar *filename);
GtkWidget *create_my_pixmap_from_file (GtkWidget *widget, const gchar *filename);
void replace_my_pixmap_from_file (GtkWidget *widget, GtkWidget *thepixmap, const gchar *filename);

/**********************************************************************************************/

#endif
